﻿using System;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 선택 마커 속성 - SelectedMarkerProperty

        /// <summary>
        /// 선택 마커 속성
        /// </summary>
        public static readonly DependencyProperty SelectedMarkerProperty = DependencyProperty.Register
        (
            "SelectedMarker",
            typeof(Shape),
            typeof(MainWindow),
            new PropertyMetadata(null)
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            Loaded += Window_Loaded;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

        /// <summary>
        /// 윈도우 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.startPointTextBox.KeyUp                   += startPointTextBox_KeyUp;
            this.endPointTextBox.KeyUp                     += endPointTextBox_KeyUp;
            this.mappingModeComboBox.SelectionChanged      += mappingModeComboBox_SelectionChanged;
            this.gradientDisplayBorder.SizeChanged         += gradientDisplayBorder_SizeChanged;
            this.gradientDisplayBorder.MouseLeftButtonDown += gradientDisplayBorder_MouseLeftButtonDown;
            this.gradientDisplayBorder.MouseMove           += gradientDisplayBorder_MouseMove;
            this.gradientDisplayBorder.MouseLeftButtonUp   += gradientDisplayBorder_MouseLeftButtonUp;
            this.linearGradientBrush.Changed               += linearGradientBrushChanged;

            startPointTextBox_KeyUp(this.startPointTextBox, null);

            endPointTextBox_KeyUp(this.endPointTextBox, null);

            linearGradientBrushChanged(this.linearGradientBrush, null);
        }

        #endregion
        #region 시작점 텍스트 박스 키 UP 처리하기 - startPointTextBox_KeyUp(sender, e)

        /// <summary>
        /// 시작점 텍스트 박스 키 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void startPointTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            TextBox textBox = sender as TextBox;

            try
            {
                Point point = Point.Parse(textBox.Text);

                if(this.linearGradientBrush.MappingMode == BrushMappingMode.RelativeToBoundingBox)
                {
                    this.startPointMarkerTranslateTransform.X = point.X * this.gradientDisplayBorder.ActualWidth;
                    this.startPointMarkerTranslateTransform.Y = point.Y * this.gradientDisplayBorder.ActualHeight;
                }
                else
                {
                    this.startPointMarkerTranslateTransform.X = point.X;
                    this.startPointMarkerTranslateTransform.Y = point.Y;
                }
            }
            catch(InvalidOperationException)
            {
            }
            catch(FormatException)
            {
            }
        }

        #endregion
        #region 중단점 텍스트 박스 키 UP 처리하기 - endPointTextBox_KeyUp(sender, e)

        /// <summary>
        /// 중단점 텍스트 박스 키 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void endPointTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            TextBox textBox = sender as TextBox;

            try
            {
                Point point = Point.Parse(textBox.Text);

                if(this.linearGradientBrush.MappingMode == BrushMappingMode.RelativeToBoundingBox)
                {
                    this.endPointMarkerTranslateTransform.X = point.X * this.gradientDisplayBorder.ActualWidth;
                    this.endPointMarkerTranslateTransform.Y = point.Y * this.gradientDisplayBorder.ActualHeight;
                }
                else
                {
                    this.endPointMarkerTranslateTransform.X = point.X;
                    this.endPointMarkerTranslateTransform.Y = point.Y;
                }
            }
            catch(InvalidOperationException)
            {
            }
            catch(FormatException)
            {
            }
        }

        #endregion
        #region 매핑 코드 콤보 박스 선택 변경시 처리하기 - mappingModeComboBox_SelectionChanged(sender, e)

        /// <summary>
        /// 매핑 코드 콤보 박스 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void mappingModeComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            Point previousStartPoint = this.linearGradientBrush.StartPoint;
            Point newStartPoint      = new Point();
            Point previousEndPoint   = this.linearGradientBrush.EndPoint;
            Point newEndPoint        = new Point();

            if(this.linearGradientBrush.MappingMode == BrushMappingMode.RelativeToBoundingBox)
            {
                newStartPoint.X = previousStartPoint.X / this.gradientDisplayBorder.ActualWidth;
                newStartPoint.Y = previousStartPoint.Y / this.gradientDisplayBorder.ActualHeight;

                this.linearGradientBrush.StartPoint = newStartPoint;

                newEndPoint.X = previousEndPoint.X / this.gradientDisplayBorder.ActualWidth;
                newEndPoint.Y = previousEndPoint.Y / this.gradientDisplayBorder.ActualHeight;

                this.linearGradientBrush.EndPoint = newEndPoint;
            }
            else
            {
                newStartPoint.X = previousStartPoint.X * this.gradientDisplayBorder.ActualWidth;
                newStartPoint.Y = previousStartPoint.Y * this.gradientDisplayBorder.ActualHeight;

                this.linearGradientBrush.StartPoint = newStartPoint;

                newEndPoint.X = previousEndPoint.X * this.gradientDisplayBorder.ActualWidth;
                newEndPoint.Y = previousEndPoint.Y * this.gradientDisplayBorder.ActualHeight;

                this.linearGradientBrush.EndPoint = newEndPoint;
            }

            this.startPointTextBox.Text = newStartPoint.X.ToString("F4") + "," + newStartPoint.Y.ToString("F4");
            this.endPointTextBox.Text   = newEndPoint.X.ToString("F4")   + "," + newEndPoint.Y.ToString("F4");
        }

        #endregion
        #region 그라디언트 표시 보더 크기 변경시 처리하기 - gradientDisplayBorder_SizeChanged(sender, e)

        /// <summary>
        /// 그라디언트 표시 보더 크기 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gradientDisplayBorder_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            if(this.linearGradientBrush.MappingMode == BrushMappingMode.RelativeToBoundingBox)
            {
                this.startPointMarkerTranslateTransform.X = this.linearGradientBrush.StartPoint.X * e.NewSize.Width;
                this.startPointMarkerTranslateTransform.Y = this.linearGradientBrush.StartPoint.Y * e.NewSize.Height;

                this.endPointMarkerTranslateTransform.X = this.linearGradientBrush.EndPoint.X * e.NewSize.Width;
                this.endPointMarkerTranslateTransform.Y = this.linearGradientBrush.EndPoint.Y * e.NewSize.Height;
            }
        }

        #endregion
        #region 그라디언트 표시 보더 마우스 왼쪽 버튼 DOWN 처리하기 - gradientDisplayBorder_MouseLeftButtonDown(sender, e)

        /// <summary>
        /// 그라디언트 표시 보더 마우스 왼쪽 버튼 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gradientDisplayBorder_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if(e.OriginalSource is Shape)
            {
                SetValue(SelectedMarkerProperty, e.OriginalSource as Shape);
            }
            else
            {
                SetValue(SelectedMarkerProperty, null);
            }
        }

        #endregion
        #region 그라디언트 표시 보더 마우스 이동시 처리하기 - gradientDisplayBorder_MouseMove(sender, e)

        /// <summary>
        /// 그라디언트 표시 보더 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gradientDisplayBorder_MouseMove(object sender, MouseEventArgs e)
        {
            Point mousePoint = e.GetPosition(this.gradientDisplayBorder);

            Shape shape = (Shape)GetValue(SelectedMarkerProperty);
            
            if(shape == this.endPointMarkerEllipse || shape == this.startPointMarkerEllipse)
            {
                TranslateTransform translationTransform = (TranslateTransform)shape.RenderTransform;

                translationTransform.X = mousePoint.X;
                translationTransform.Y = mousePoint.Y;

                Mouse.Synchronize();

                Point point;

                if(this.linearGradientBrush.MappingMode == BrushMappingMode.RelativeToBoundingBox)
                {
                    point = new Point
                    (
                        mousePoint.X / gradientDisplayBorder.ActualWidth,
                        mousePoint.Y / gradientDisplayBorder.ActualHeight
                    );
                }
                else
                {
                    point = mousePoint;
                }

                if(shape == this.startPointMarkerEllipse)
                {
                    this.linearGradientBrush.StartPoint = point;

                    this.startPointTextBox.Text = point.X.ToString("F4") + "," + point.Y.ToString("F4");
                }
                else
                {
                    this.linearGradientBrush.EndPoint = point;

                    this.endPointTextBox.Text = point.X.ToString("F4") + "," + point.Y.ToString("F4");
                }
            }
        }

        #endregion
        #region 그라디언트 표시 보더 마우스 왼쪽 버튼 UP 처리하기 - gradientDisplayBorder_MouseLeftButtonUp(sender, e)

        /// <summary>
        /// 그라디언트 표시 보더 마우스 왼쪽 버튼 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gradientDisplayBorder_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            Point mousePoint = e.GetPosition(this.gradientDisplayBorder);

            Shape shape = (Shape)GetValue(SelectedMarkerProperty);

            if(shape == this.endPointMarkerEllipse || shape == this.startPointMarkerEllipse)
            {
                TranslateTransform translationTransform = shape.RenderTransform as TranslateTransform;

                translationTransform.X = mousePoint.X;
                translationTransform.Y = mousePoint.Y;

                SetValue(SelectedMarkerProperty, null);

                Mouse.Synchronize();

                Point point;

                if(this.linearGradientBrush.MappingMode == BrushMappingMode.RelativeToBoundingBox)
                {
                    point = new Point
                    (
                        mousePoint.X / gradientDisplayBorder.ActualWidth,
                        mousePoint.Y / gradientDisplayBorder.ActualHeight
                    );
                }
                else
                {
                    point = mousePoint;
                }

                if(shape == this.startPointMarkerEllipse)
                {
                    this.linearGradientBrush.StartPoint = point;

                    this.startPointTextBox.Text = point.X.ToString("F4") + "," + point.Y.ToString("F4");
                }
                else
                {
                    this.linearGradientBrush.EndPoint = point;

                    this.endPointTextBox.Text = point.X.ToString("F4") + "," + point.Y.ToString("F4");
                }
            }
        }

        #endregion
        #region 선형 그라디언트 브러시 변경시 처리하기 - linearGradientBrushChanged(sender, e)

        /// <summary>
        /// 선형 그라디언트 브러시 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void linearGradientBrushChanged(object sender, EventArgs e)
        {
            if(this.gradientDisplayBorder != null)
            {
                this.markupOutputTextBlock.Text = GetLinearGradientBrushMarkup(this.linearGradientBrush);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 선형 그라디언트 브러시 마크업 구하기 - GetLinearGradientBrushMarkup(brush)

        /// <summary>
        /// 선형 그라디언트 브러시 마크업 구하기
        /// </summary>
        /// <param name="brush">브러시</param>
        /// <returns>선형 그라디언트 브러시 마크업</returns>
        private static string GetLinearGradientBrushMarkup(LinearGradientBrush brush)
        {
            StringBuilder stringBuilder = new StringBuilder();

            stringBuilder.AppendLine($"<LinearGradientBrush");
            stringBuilder.AppendLine($"    StartPoint=\"{brush.StartPoint}\"");
            stringBuilder.AppendLine($"    EndPoint=\"{brush.EndPoint}\"");
            stringBuilder.AppendLine($"    MappingMode=\"{brush.MappingMode}\"");
            stringBuilder.AppendLine($"    SpreadMethod=\"{brush.SpreadMethod}\"");
            stringBuilder.AppendLine($"    ColorInterpolationMode=\"{brush.ColorInterpolationMode}\"");
            stringBuilder.AppendLine($"    Opacity=\"{brush.Opacity}\">");

            foreach(GradientStop stop in brush.GradientStops)
            {
                stringBuilder.AppendLine($"    <GradientStop Offset=\"{stop.Offset.ToString("F4")}\" Color=\"{stop.Color}\" />");
            }

            stringBuilder.Append("</LinearGradientBrush>");

            return stringBuilder.ToString();
        }

        #endregion
    }
}